classdef Data < handle
  % Data
  %   This is the class that contains depth or image data
  %
  % Methods:
  %   data - Get pointer to data
  %   Size - Get size of the data
  %   getUserData - Get user data that will get saved to csf file
  %   setUserData - Set user data that will get saved to csf file
  %   matType - Get the matrix type
  %   rows - Get the number of rows
  %   cols - Get the number of columns
  %   frameCount - The frame counter
  %   setFrameCount - Set the frame counter
  %   frameType - Get the frame type
  %   setFrameType - Set the frame type
  %   frameId - Get the frame id
  %   setFrameId - Set the frame id
  %   modulationFrequency - Get the modulation frequency of the frame
  %   setModulationFrequency - Set the modulation frequency of the frame
  %   integrationTime - Get the integration time of the frame
  %   setIntegrationTime - Set the integration time of the frame
  %   process - Get a bit array of the processing performed on the frame
  %   setProcess - Set a bit array of the processing performed on the frame
  %   scale - Get the scale used for integer frames, to get the original value
  %           divide by scale
  %   setScale - Set the scale used for integer frames, to get the original
  %              value divide by scale
  %   addition - Get the addition used for integer frames, to get the original
  %              value substract the addition
  %   setAddition - Set the addition used for integer frames, to get the
  %                 original value substract the addition
  %   temperatures - Get the temperature of various parts of the camera
  %   setTemperatures - Set the temperature of various parts of the camera
  %   syncCount - Get the synchronization count
  %   ambientIr - Get the readings from the ambient infrared sensor
  %   cameraStatus - Get the camera status of the kea camera
  %   inputState - Get the state of the digital inputs
  %   userAllocated - Check whether the data uses a pointer supplied by the user
  %   userPointer - The user data that was given in the add_user_pointer
  %                 function
  %   acceleration - The acceleration measured by the imu
  %   setAcceleration - Set the acceleration measured by the imu
  %   angularVelocity - The angular velocity measured by the imu
  %   setAngularVelocity - Set the angular velocity measured by the imu
  %   maxCurrent - Maximum current measured over 20 samples
  %   setMaxCurrent - Set maximum current
  %

  properties(Access=protected)
    mPtr = uint64(0);
  end

  methods(Access=public)
    function obj = Data(varargin)
      % Data
      %   Construct new data class
      %
      % ----------
      %
      %   Create new tof data
      %
      % Inputs:
      %   1 - numeric - Number of rows
      %   2 - numeric - Number of columns
      %   3 - MatType - The matrix type
      %

      if nargin == 1 && isa(varargin{1}, 'uint64')
        obj.mPtr = varargin{1};
      else
        error("Can't construct empty object");
      end
    end


    function delete(obj)
      if obj.mPtr ~= 0
        tof.mex('tof_data_delete', obj.mPtr);
        obj.mPtr = 0;
      end
    end

    function ptr = get_object_ptr(obj)
      ptr = obj.mPtr;
    end

% 
    function ret_val = data(obj)
      % data
      %   Return a matlab matrix containing the image data. Due to 
      %   matlab internals this function can only be called once
      %
      % Output:
      %   ret_val - numeric - Matrix of the data
      %

      ret_val = tof.mex('tof_data_data', obj.mPtr);
    end

    function ret_val = Size(obj)
      % Size
      %   Get size of the data
      %
      % Output:
      %   ret_val - numeric - The size
      %

      ret_val = tof.mex('tof_data_size', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = getUserData(obj)
      % getUserData
      %   Get user data that will get saved to csf file
      %
      % Output:
      %   ret_val - numeric - Set user data
      %

      ret_val = tof.mex('tof_data_get_user_data', obj.mPtr);
      ret_val = double(ret_val);
    end

    function setUserData(obj, user_data)
      % setUserData
      %   Set user data that will get saved to csf file
      %
      % Inputs:
      %   user_data - numeric - Set user data
      %

      tof.mex('tof_data_set_user_data', obj.mPtr, user_data);
    end

    function ret_val = matType(obj)
      % matType
      %   Get the matrix type
      %
      % Output:
      %   ret_val - MatType - The matrix type
      %

      ret_val = tof.mex('tof_data_mat_type', obj.mPtr);
    end

    function ret_val = rows(obj)
      % rows
      %   Get the number of rows
      %
      % Output:
      %   ret_val - numeric - Number of rows
      %

      ret_val = tof.mex('tof_data_rows', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = cols(obj)
      % cols
      %   Get the number of columns
      %
      % Output:
      %   ret_val - numeric - Number of columns
      %

      ret_val = tof.mex('tof_data_cols', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = frameCount(obj)
      % frameCount
      %   The frame counter
      %
      % Output:
      %   ret_val - numeric - Frame count
      %

      ret_val = tof.mex('tof_data_frame_count', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = frameType(obj)
      % frameType
      %   Get the frame type
      %
      % Output:
      %   ret_val - FrameType - The frame type
      %

      ret_val = tof.mex('tof_data_frame_type', obj.mPtr);
    end

    function ret_val = frameId(obj)
      % frameId
      %   Get the frame id
      %
      % Output:
      %   ret_val - numeric - The frame id
      %

      ret_val = tof.mex('tof_data_frame_id', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = modulationFrequency(obj)
      % modulationFrequency
      %   Get the modulation frequency of the frame
      %
      % Output:
      %   ret_val - numeric - The modulation frequency
      %

      ret_val = tof.mex('tof_data_modulation_frequency', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = integrationTime(obj)
      % integrationTime
      %   Get the integration time of the frame
      %
      % Output:
      %   ret_val - numeric - The integration time
      %

      ret_val = tof.mex('tof_data_integration_time', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = process(obj)
      % process
      %   Get a bit array of the processing performed on the frame
      %
      % Output:
      %   ret_val - numeric - A bit array
      %

      ret_val = tof.mex('tof_data_process', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = scale(obj)
      % scale
      %   Get the scale used for integer frames, to get the original value
      %   divide by scale
      %
      % Output:
      %   ret_val - numeric - The scale
      %

      ret_val = tof.mex('tof_data_scale', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = addition(obj)
      % addition
      %   Get the addition used for integer frames, to get the original value
      %   substract the addition
      %
      % Output:
      %   ret_val - numeric - The addition
      %

      ret_val = tof.mex('tof_data_addition', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = temperatures(obj)
      % temperatures
      %   Get the temperature of various parts of the camera
      %
      % Output:
      %   ret_val - vector[numeric] - Temperatures
      %

      ret_val = tof.mex('tof_data_temperatures', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = syncCount(obj)
      % syncCount
      %   Get the synchronization count
      %
      % Output:
      %   ret_val - numeric - Sync count
      %

      ret_val = tof.mex('tof_data_sync_count', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = userAllocated(obj)
      % userAllocated
      %   Check whether the data uses a pointer supplied by the user
      %
      % Output:
      %   ret_val - logical - Data allocated by user
      %

      ret_val = tof.mex('tof_data_user_allocated', obj.mPtr);
    end

    function ret_val = userPointer(obj)
      % userPointer
      %   The user data that was given in the add_user_pointer function
      %
      % Output:
      %   ret_val - numeric - User pointer
      %

      ret_val = tof.mex('tof_data_user_pointer', obj.mPtr);
    end

    function ret_val = acceleration(obj)
      % acceleration
      %   The acceleration measured by the imu
      %
      % Output:
      %   ret_val - array[numeric] - Acceleration, x y z
      %

      ret_val = tof.mex('tof_data_acceleration', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = angularVelocity(obj)
      % angularVelocity
      %   The angular velocity measured by the imu
      %
      % Output:
      %   ret_val - array[numeric] - Angular velocity, x y z
      %

      ret_val = tof.mex('tof_data_angular_velocity', obj.mPtr);
      ret_val = double(ret_val);
    end

  end
end
